// Astronomer's Pal
// Calculates your local sidereal time
// as well as the required azimuth and elevation to point
// your telescope, given its right ascension and declination.

// version 0.1

// by LUKE HAYWAS, UNITED STATES AIR FORCE ACADEMY
// cadet squadron 28, class of 2005
// c05luke.haywas@usafa.af.mil


// last update 20 November 2002

// Do not steal, modify, or illegally redistribute this program.


#define USE_TI89              // Produce .89z File
#define USE_TI92PLUS          // Produce .9xz File

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files



// don't use this program before the year 2000
// well it is now 2002 so who cares about that


// all the important crap is in here
void _main(void)
{
	
	int year, month, day, yi, mi, di;
	int hour, minute, second, hi, ni;
	float h, m, s;
	float yr;
	float longitude;
	char q[20];
	int numdays[13] = {31, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	char mname[13][4] = {{"---\000"}, {"JAN\000"}, {"FEB\000"}, {"MAR\000"}, {"APR\000"}, {"MAY\000"}, {"JUN\000"}, {"JUL\000"}, {"AUG\000"}, {"SEP\000"}, {"OCT\000"}, {"NOV\000"}, {"DEC\000"}};
				// all the zeros append a null character to the end of every little string, to make
				// it possible to display this as an array of strings

	float daysince;
	int daywhole;
	float daypart;
	int a, b, c;		// integer random use variables
	float d;			// floating random use variable
	int timezone;
	float lst, zst;
	float radst, radlat, radlong, radra, raddec;
	float ra, dec, lat, ha, azimuth, elev;

	// initial stuff
	ClrScr();
	MoveTo(0, 0);
	puts("Astronomer's Pal\n");
	puts("Luke Haywas, USAFA '05\n\n");
	
	// get info
	puts("Year     :");
	getstr(q, 20);
	year = atoi(q);
	
	c = PopupNew("Month", 80);
	for(a = 1; a <= 12; a++)
		PopupAddText(c, -1, mname[a], a);
	month = PopupDo(c, CENTER, CENTER, 0);	
	
	
	
	
	puts("Day      :");
	getstr(q, 20);
	day = atoi(q);
	
	puts("Longitude:");
	getstr(q, 20);
	longitude = atof(q);		// negative if west...
	puts("Latitude :");
	getstr(q, 20);
	lat = atof(q);				// negative if south...
	
	radlat = lat * PI / 180;
	radlong = longitude * PI / 180;
	
	
	puts("Hour     :");			// hour must be in 24 hour format standard time, if you
	getstr(q, 20);				// use daylight saving or am/pm you are a dumbass and deserve
	hour = atoi(q);				// to have this program give you the wrong answer
	puts("Minute   :");
	getstr(q, 20);
	minute = atoi(q);
	puts("Second   :");
	getstr(q, 20);
	second = atoi(q);
	
	
	PopupClear(c);
	c = PopupNew("Time Zone", 80);
	PopupAddText(c, -1, "Greenwich 0", 24);
	PopupAddText(c, -1, "Eastern  -5", 19);
	PopupAddText(c, -1, "Central  -6", 18);
	PopupAddText(c, -1, "Mountain -7", 17);
	PopupAddText(c, -1, "Pacific  -8", 16);
	timezone = -24 + PopupDo(c, CENTER, CENTER, 0);
	
	ClrScr();
	MoveTo(0, 0);
	
	puts("Celestial Body\n     Coordinates\n\n");
	puts("Right Ascension (deg):\n");
	getstr(q, 20);
	ra = atof(q);
	puts("Declination (deg):\n");
	getstr(q, 20);
	dec = atof(q);
	
	radra = ra * PI / 180;
	raddec = dec * PI / 180;
	
	hi = hour;
	ni = minute;
	yi = year;
	mi = month;
	di = day;
		
	
	FontSetSys(F_8x10);
	DrawStr(30, 35, "CRUNCHING", A_REVERSE);
	FontSetSys(F_6x8);
		
	
	d = year - 2000;
	if(year % 4 == 0) numdays[2]++;
	
	hour -= timezone;			// change it to zulu
	
	if(hour < 0)
	{
		hour += 24;
		day--;
		if(day < 1)
		{
			if(month == 1)
				day = 31;
			else
				day = numdays[month - 1];
			month--;
			if(month == 0)
			{
				month = 12;
				year--;
			}
		}
	}
	
	if(hour > 23)
	{
		hour -= 24;
		day++;
		if(day > numdays[month])
		{
			day = 1;
			month++;
			if(month == 13)
			{
				month = 1;
				year++;
			}
		}
	}
	
	if(numdays[2] != 28)
		numdays[2] = 28;

h = hour;
m = minute;
s = second;

yr = year;

daywhole = 0;

for(a = 2000; a < year; a++)				// see how many years have elapsed since J2000
{
	daywhole += 365;						// add 365 days for each year
	if(a % 4 == 0) daywhole++;				// plus one if it's divisible by four (leap year)
}											// if anyone uses this after 2100, you will be a day off.

for(a = 1; a < month; a++) daywhole += numdays[a];		// see how many days have elapsed in this year
if((year % 4 == 0) && (month > 2)) daywhole++;				// add one if leap year, after February

daywhole += (day - 1);										// add the day of the current month
															// minus one because this day hasn't passed yet
daypart = ((h + m / 60 + s / 3600) / 24);				// add the fraction of the current day

d = daywhole;											// make it float

daysince = d + daypart - 0.5;		// minus a half because J2000 is at 1200 hours on 1 Jan 2000
	
	zst = 280.46061837 + 360.98564736629 * daysince;	// Greenwich mean sidereal time
	lst = zst + longitude;								// Local sidereal time
		
	// normalize it zero to 360
	
	while(lst > 360)
		lst -= 360;
	
	radst = lst * PI / 180;			// make it radians for the altazimuth calculator
	
	// convert to hours, minutes
	
	lst /= 360;		// converts to fraction of a day
	lst *= 24;		// converts to hours
	
	a = floor(lst);		// gets whole number of hours
	d = lst - floor(lst);	// gets fraction of minutes
	b = 60 * d;				// gets whole number of minutes
	
	
	// ok now that we have the info, let's calculate altazimuth.
	
	ha = radst - radra;		// calc the hour angle
	if(ha > (2 * PI)) ha -= (2 * PI);	// normalize
	
	elev = sin(raddec) * sin(radlat) + cos(raddec) * cos(radlat) * cos(ha);
	elev = asin(elev);   // elevation...
	
	azimuth = (sin(raddec) - sin(elev) * sin(radlat)) / (cos(elev) * cos(radlat));
	azimuth = acos(azimuth);		// azimuth...
	
	elev *= (180 / PI);				// make degrees
	azimuth *= (180 / PI);			// make degrees
		
	ClrScr();
	MoveTo(0,0);
	
	// spit it out and go home
	
	// let's now make a nice display page with relevant items emphasized.
	FontSetSys(F_6x8);
	DrawStr(0, 0, "Astronomer's Pal v0.1", A_NORMAL);
	FontSetSys(F_4x6);
	
	DrawStr(0, 11, "By Luke Haywas, USAFA 2005", A_NORMAL);
	DrawLine(0, 18, 158, 18, A_NORMAL);
	
	DrawStr(25, 20, "Time", A_NORMAL);
	DrawStr(62, 20, "Date", A_NORMAL);
	DrawStr(0, 30, "Local", A_NORMAL);
	DrawStr(0, 41, "Zulu", A_NORMAL);
	
	sprintf(q, "%d:%d", hi, ni);
	FontSetSys(F_6x8);
	DrawStr(25, 28, q, A_NORMAL);
	sprintf(q, "%d %s %d", di, mname[mi], yi);
	DrawStr(62, 28, q, A_NORMAL);
	sprintf(q, "%d:%d", hour, minute);
	DrawStr(25, 39, q, A_NORMAL);
	sprintf(q, "%d %s %d", day, mname[month], year);
	DrawStr(62, 39, q, A_NORMAL);
	
	DrawLine(0, 47, 158, 47, A_NORMAL);
	DrawLine(21, 18, 21, 47, A_NORMAL);
	DrawLine(60, 18, 60, 47, A_NORMAL);
	
	sprintf(q, "%3.1f Lat", lat);
	FontSetSys(F_4x6);
	DrawStr(0, 49, q, A_NORMAL);
	sprintf(q, "%3.1f Long", longitude);
	DrawStr(0, 56, q, A_NORMAL);
	DrawLine(0, 63, 158, 63, A_NORMAL);
	
	DrawStr(0, 65, "Right Asc", A_NORMAL);
	DrawStr(0, 81, "Declin", A_NORMAL);
	FontSetSys(F_6x8);
	sprintf(q, "%3.1f", ra);
	DrawStr(0, 72, q, A_NORMAL);
	sprintf(q, "%3.1f", dec);
	DrawStr(0, 89, q, A_NORMAL);
	
	FontSetSys(F_4x6);
	
	DrawStr(52, 49, "Sidereal", A_NORMAL);
	DrawStr(52, 56, "Time", A_NORMAL);
	FontSetSys(F_6x8);
	sprintf(q, "%d:%d", a, b);
	DrawStr(100, 51, q, A_NORMAL);
	
	DrawLine(42, 47, 42, 98, A_NORMAL);
	
	FontSetSys(F_4x6);
	DrawStr(44, 70, "Azimuth", A_NORMAL);
	DrawStr(44, 86, "Elevation", A_NORMAL);
	sprintf(q, "%3.1f", azimuth);
	FontSetSys(F_8x10);
	DrawStr(80, 67, q, A_NORMAL);
	sprintf(q, "%3.1f", elev);
	DrawStr(80, 84, q, A_NORMAL);
	
	
	
	
	
	// the following was the old, debugging display
	/*
	printf("\nDays Since J2000.0:\n%f\n", daysince);
	printf("Zulu Time = %d:%d\n", hour, minute);
	printf("Zulu Date = %d/%d/%d\n", month, day, year);
	printf("Local\n Sidereal Time = %d:%d\n", a, b);
	printf("Az = %f\nEl = %f\n", azimuth, elev);
	getchar();
	*/
	
	getchar();
	
}

// bye bye

